// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef AUDIO_HPP
#define AUDIO_HPP

#include <SDL_mixer.h>

class Audio{
	public:
		// initializes the audio [depends on: Config]
		static void init();
		// reinitializes the audio, call it when config changes
		static void reinit();
		// ends the audio
		static void finalize();

		// stops the audio
		static void stop();

		// plays the sound id
		static void playSound( int id, bool silent = false );
		// stops the sound id (to stop the SOUND_REDHOLE_DRAG, stopSound must be called as
		// many times as startSound was called)
		static void stopSound( int id );

		// identifiers for the sounds
		enum soundnames {
			SOUND_BOMB = 0,
			SOUND_BULLET_FIRE,
			SOUND_BULLET_HIT,
			SOUND_ENEMY_BIRTH,
			SOUND_GAME_OVER,
			SOUND_GAME_START,
			SOUND_LASER_FIRE,
			SOUND_LIGHTNING_FIRE,
			SOUND_MENU_ACCEPT,
			SOUND_MENU_CHANGE,
			SOUND_MENU_EXIT,
			SOUND_MENU_MOVE,
			SOUND_PLAYER_DEATH,
			SOUND_REDHOLE_DRAG,

			SOUND_AMOUNT
		};

	private:
		// audio file loader
		static void loadSounds();

		// callback for SDL_mixer when a channel ends
		static void audiocallback( int id );

		// the module has been initialized
		static bool initialized;

		// all sounds data
		static Mix_Chunk *sound_data[SOUND_AMOUNT];

		// number of active redholes
		static int activeRedHole;
		// true if sound is playing
		static bool playing[SOUND_AMOUNT];
};

#endif // Audio.hpp
