// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef LOCALE_HPP
#define LOCALE_HPP

#include <set>
#include <map>
#include <string>
#include <vector>

// GUI and Menu default language
#define DEFAULT_LANG "English"

class Locale{
	public:
		// preinitializes the locale [depends on: -]
		static void preinit();
		// initializes the locale [depends on: Config]
		static void init();
		// reinitializes the locale, call it when config changes
		static void reinit();
		// ends the locale
		static void finalize();

		// gets the string corresponding to the code
		static const std::string& get( int code );
		// gets the available languages
		// always at least one: the present language
		static const std::set<std::string>& get_availablelangs();

		// possible texts' identifier code
		enum textcode {
			// GUI strings
			SCORE_STR = 0,
			BOMBS_STR,
			HISCORE_STR,
			GAMEOVER_STR,
			// configuration strings
			CONFIG_STR_GRID,
			CONFIG_STR_GRIDBEZIER,
			CONFIG_STR_DEBRIS,
			CONFIG_STR_AUDIO,
			CONFIG_STR_RESOLUTION,
			CONFIG_STR_BITDEPTH,
			CONFIG_STR_FULLSCREEN,
			CONFIG_STR_LINESMOOTH,
			// controls strings
			CONTROLS_STR_PLAYER,
			CONTROLS_STR_KEYBOARD,
			CONTROLS_STR_KESCAPE,
			CONTROLS_STR_KBOMB,
			CONTROLS_STR_KUP,
			CONTROLS_STR_KDOWN,
			CONTROLS_STR_KRIGHT,
			CONTROLS_STR_KLEFT,
			CONTROLS_STR_KFIREUP,
			CONTROLS_STR_KFIREDOWN,
			CONTROLS_STR_KFIRERIGHT,
			CONTROLS_STR_KFIRELEFT,
			CONTROLS_STR_JOYSTICK,
			CONTROLS_STR_JESCAPE,
			CONTROLS_STR_JBOMB,
			CONTROLS_STR_JUPDOWN,
			CONTROLS_STR_JRIGHTLEFT,
			CONTROLS_STR_JFIREUPDOWN,
			CONTROLS_STR_JFIRERIGHTLEFT,
			// root strings
			ROOT_STR_RESTART,
			ROOT_STR_SAVE,
			ROOT_STR_LOAD,
			ROOT_STR_CONFIG,
			ROOT_STR_CONTROLS,
			ROOT_STR_EXIT,
			// misc strings
			STR_OK,
			STR_ON,
			STR_OFF,

			STR_AMOUNT
		};

	private:
		// loads the locale data, best-effort: it loads as much as it can
		static void load();

		// every single language data available
		static std::map< std::string, std::vector<std::string> > langs; // unordered
		// available languages
		// always at least one: the present language
		static std::set<std::string> available;
};

#endif // Locale.hpp
