// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include "../rc/Constants.hpp"
#include "../vis/Draw.hpp"
#include "../Audio.hpp"
#include "../Main.hpp"
#include "Bullet.hpp"

// ctor
au::Bullet::Bullet( const util::Vector& xy, const util::Vector& v, int lifes, bool play ) :
 Automaton( xy, v, rc::bullet_r() ),
 lifes( lifes )
{
	start_activecollider( SHOT_COL );
	start_passivecollider( EXAMINIS_COL | REDHOLE_COL | SHIELDED_COL );
	start_gravitylistener();
	start_processable();
	start_shot();
	start_visible( BULLET_PRIO );

	if ( play )
		Audio::playSound( Audio::SOUND_BULLET_FIRE );
}
// dtor
au::Bullet::~Bullet(){
	if ( dolitter )
		Automaton::create_debris( xy, rc::bullet_litter_r(), rc::bullet_litter_c_r(), rc::bullet_litter_c_g(), rc::bullet_litter_c_b() );

	Audio::playSound( Audio::SOUND_BULLET_HIT );
}
// bouncing function
void au::Bullet::bounce( const util::Vector& newv ){
	v = newv;
}
// each collision type for active colliders
void au::Bullet::recieve_collision( int& res, Lagvard *p ){
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
void au::Bullet::recieve_collision( int& res, Proton *p ){
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
void au::Bullet::recieve_collision( int& res, Scared *p ){
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
void au::Bullet::recieve_collision( int& res, Spawned *p ){
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
void au::Bullet::recieve_collision( int& res, Spawnship *p ){
	p->do_spawn();
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
// serializes the automaton
void au::Bullet::serialize( std::ostream& out ){
	out << Automaton::BULLET_SERIAL
	    << " " << xy << " " << v
	    << " " << lifes << std::endl;
}
// unserializes the automaton
void au::Bullet::unserialize( std::istream& in ){
	util::Vector xy;
	util::Vector v;
	int lifes;

	in >> xy >> v
	   >> lifes;
	if ( lifes <= 0 )
		throw Main::UNSERIAL_BULLET_LIFES;

	new Bullet( xy, v, lifes, false );
}
// drawing function
void au::Bullet::specific_draw(){
	vis::Draw::translate( xy );
	vis::Draw::rotate( v.angle() );
	vis::Draw::play_record( vis::BULLET_ID );
}
// process function
void au::Bullet::specific_process( int milliseconds ){
	xy += v*milliseconds + g*(milliseconds*milliseconds/2.0);

	v += g * milliseconds;

	// we will litter only if it is the ring who kills the bullet
	dolitter = checkRing_die();
}
// collision functions
void au::Bullet::send_collision( int& res, Automaton *a ){
	a->recieve_collision( res, this );
}
