// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include <algorithm>
#include "../rc/Constants.hpp"
#include "../vis/Draw.hpp"
#include "../Main.hpp"
#include "Debris.hpp"

// ctor
au::Debris::Debris( const util::Vector& xy, const util::Vector& v, real red, real green, real blue, real alpha ) :
 Automaton( xy, v, rc::debris_r() ),
 red( red ),
 green( green ),
 blue( blue ),
 alpha( alpha )
{
	start_passivecollider( REDHOLE_COL | BLAST_COL );
	start_gravitylistener();
	start_processable();
	start_visible( DEBRIS_PRIO );
}
// dtor
au::Debris::~Debris(){
}
// color getters
real au::Debris::get_red(){
	return red;
}
real au::Debris::get_green(){
	return green;
}
real au::Debris::get_blue(){
	return blue;
}
real au::Debris::get_alpha(){
	return alpha;
}
// serializes the automaton
void au::Debris::serialize( std::ostream& out ){
	out << Automaton::DEBRIS_SERIAL
	    << " " << xy << " " << v
	    << " " << red << " " << green << " " << blue << " " << alpha << std::endl;
}
// unserializes the automaton
void au::Debris::unserialize( std::istream& in ){
	util::Vector xy;
	util::Vector v;
	real red;
	real green;
	real blue;
	real alpha;

	in >> xy >> v
	   >> red >> green >> blue >> alpha;
	if ( (alpha < 0) || (alpha > 1) )
		throw Main::UNSERIAL_DEBRIS;

	new Debris( xy, v, red, green, blue, alpha );
}
// drawing function
void au::Debris::specific_draw(){
	util::Vector _xy = xy - v*rc::debris_milliseconds_ago();

	vis::Draw::set_thin_lines();
	vis::Draw::set_color( red, green, blue, alpha );
	vis::Draw::start_lines();
		vis::Draw::add_point( _xy );
		vis::Draw::add_point(  xy );
	vis::Draw::end_lines();
}
// process function
void au::Debris::specific_process( int milliseconds ){
	if ( g.magnitude() < rc::debris_g_threshold() ){
		// if not enough gravity recieved, then we are dying
		alpha *= std::max( 0.0, 1.0-rc::debris_fade_out()*milliseconds );
		if ( alpha < rc::debris_min_visiblity() ){
			add_zombie();
			return;
		}
	}else
		alpha = std::min( 1.0, alpha*(1.0 + rc::debris_fade_out()*milliseconds) );

	xy += v*milliseconds + g*(milliseconds*milliseconds/2.0);

	v += g * milliseconds;
	v *= 1.0 - rc::debris_s_lose()*milliseconds; // lineally decreasing speed

	checkRing_die();
}
// collision functions
void au::Debris::send_collision( int& res, Automaton *a ){
	a->recieve_collision( res, this );
}
