// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef GRID_HPP
#define GRID_HPP

#include "../util/Vector.hpp"
#include "../util/Matrix.hpp"
#include "Automaton.hpp"

namespace au {

class Grid{
	public:
		// initializes the grid [depends on: Config]
		static void init();
		// reinitializes the grid, call it when config changes
		static void reinit();
		// ends the grid
		static void finalize();

		// serializes the grid
		static void serialize( std::ostream& out );
		// postoperations when the serialization has finished
		static void end_serialize();
		// unserializes the grid
		static void unserialize( std::istream& in );

		// drawing function
		static void draw();

	private:
		// draw grid using lines
		static void drawlines();
		// draw grid using bezier curves
		static void drawbezier();
		// compute bezier control points for act
		static void calcPoints( const util::Vector& prev,
		                        const util::Vector& act,
		                        const util::Vector& next,
		                        const real rad,
		                        util::Vector& actA,
		                        util::Vector& actB );

		// the grid
		static util::Matrix<Spring*> grid;
		static int rows;
		static int cols;

		// deletes the grid
		static void deletegrid();
};

}

#endif // Grid.hpp
