// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include "../rc/Constants.hpp"
#include "../vis/Draw.hpp"
#include "../Main.hpp"
#include "Lagvard.hpp"

// ctor
au::Lagvard::Lagvard( const util::Vector& xy, int t, bool play ) :
 Automaton( xy, util::Vector(0.0,0.0), rc::lagvard_r(), t )
{
	m = rc::lagvard_mass();

	start_passivecollider( PLAYER_COL | SHOT_COL | REDHOLE_COL );
	start_enemy( play );
	start_gravitylistener();
	start_processable();
	start_pursuiter();
	start_visible( LAGVARD_PRIO );
}
// dtor
au::Lagvard::~Lagvard(){
	if ( dolitter )
		Automaton::create_debris( xy, rc::lagvard_litter_r(), rc::lagvard_litter_c_r(), rc::lagvard_litter_c_g(), rc::lagvard_litter_c_b() );
}
// serializes the automaton
void au::Lagvard::serialize( std::ostream& out ){
	out << Automaton::LAGVARD_SERIAL
	    << " " << xy << " " << t << std::endl;
}
// unserializes the automaton
void au::Lagvard::unserialize( std::istream& in ){
	util::Vector xy;
	int t;

	in >> xy >> t;
	if ( t < 0 )
		throw Main::UNSERIAL_LAGVARD;

	new Lagvard( xy, t, false );
}
// drawing function
void au::Lagvard::specific_draw(){
	vis::Draw::translate( xy );
	if ( t < rc::lagvard_t() )
		vis::Draw::scale( 1.0+(rc::lagvard_t()-t)*rc::lagvard_ds() );
	vis::Draw::play_record( vis::LAGVARD_ID );
}
// where is the nearest player. itis is an unit vector or zero.
// distance is the distance or REAL_MAX
void au::Lagvard::specific_there( const util::Vector& itis, real distance ){
	v = itis * rc::lagvard_s();
}
// process function
void au::Lagvard::specific_process( int milliseconds ){
	if ( t > rc::lagvard_t() )
		xy += v*milliseconds + g*(milliseconds*rc::non_inertial_time_multiplier());

	checkRing_stop();
}
// collision functions
void au::Lagvard::send_collision( int& res, Automaton *a ){
	a->recieve_collision( res, this );
}
// scoring points
int au::Lagvard::get_killscore() const{
	return rc::lagvard_kill_score();
}
