// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef LASER_HPP
#define LASER_HPP

#include "Automaton.hpp"
#include "LaserParts.hpp"

namespace au {

// laser fired by the user
class Laser : public Automaton{
	public:
		// ctor
		Laser( const util::Vector& xy, const util::Vector& v, const util::Vector& v2, int lifes = rc::laser_lifes(), bool play = true );
		// dtor
		virtual ~Laser();

		// unserializes the automaton
		static void unserialize( std::istream& in );

	private:
		// lifes
		int lifes;
		// origin
		util::Vector o;
		// origin speed
		util::Vector v2;

		// parts to draw
		LaserParts *parts;

	// redefined automaton functions

	public:
		// bouncing function
		virtual void bounce( const util::Vector& newv );
		// each collision type for active colliders
		virtual void recieve_collision( int& res, Lagvard *p );
		virtual void recieve_collision( int& res, Proton *p );
		virtual void recieve_collision( int& res, Scared *p );
		virtual void recieve_collision( int& res, Spawned *p );
		virtual void recieve_collision( int& res, Spawnship *p );
		// serializes the automaton
		virtual void serialize( std::ostream& out );

	protected:
		// drawing function
		virtual void specific_draw();
		// process function
		virtual void specific_process( int milliseconds );
		// collision functions
		virtual void send_collision( int& res, Automaton *a );
};

}

#endif // Laser.hpp
