// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef LASERPARTS_HPP
#define LASERPARTS_HPP

#include <list>
#include <istream>
#include <ostream>
#include "../util/Vector.hpp"

// prototype
namespace au { class LaserParts; }

// formatted io
extern std::ostream& operator<<( std::ostream& out, const au::LaserParts& lp );
extern std::istream& operator>>( std::istream& in, au::LaserParts& lp );

namespace au {

// auxiliar class for lasers
class LaserParts{
	public:
		// ctor
		LaserParts( real v_head = 0.0, real v_tail = 0.0 );
		// dtor
		virtual ~LaserParts();

		// starts a part in the laser
		void pushpart( const util::Vector& xy, real vtheta );

		// there are no parts?
		bool empty() const;

		// processing function, updates the tail parameters as needed
		void specific_process( int milliseconds, util::Vector& tail, util::Vector& vtail );

		// drawing function
		void specific_draw();

	private:
		// speed of the head
		real v_head;
		// speed of the tail
		real v_tail;
		// global growing speed
		real v_grow;

		// full length
		real l_full;

		// needed information for each stop point
		typedef struct {
			real left; // length of the tail from this stop point
			util::Vector xy; // where is the stop point located
			real vtheta; // direction that the head of the laser took
		} laserpart_t;
		// laser parts' list
		std::list<laserpart_t> parts;

	// friendly formatted io
	friend std::ostream& ::operator<<( std::ostream& out, const au::LaserParts& lp );
	friend std::istream& ::operator>>( std::istream& in, au::LaserParts& lp );
};

}

#endif // LaserParts.hpp
