// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include "../rc/Constants.hpp"
#include "../vis/Draw.hpp"
#include "LaserTail.hpp"

// ctor
au::LaserTail::LaserTail( const util::Vector& xy, const util::Vector& v2, LaserParts *parts ) :
 Automaton( xy, v2, rc::laser_r() ),
 parts( parts )
{
	start_processable();
	start_visible( LASERTAIL_PRIO );
}
// dtor
au::LaserTail::~LaserTail(){
	delete parts;
}
// serializes the automaton
void au::LaserTail::serialize( std::ostream& out ){
	out << Automaton::LASERTAIL_SERIAL
	    << " " << xy << " " << v
	    << " " << (*parts) << std::endl;
}
// unserializes the automaton
void au::LaserTail::unserialize( std::istream& in ){
	util::Vector xy;
	util::Vector v;

	in >> xy >> v;

	LaserParts *parts = new LaserParts();
	new LaserTail( xy, v, parts );

	in >> (*parts);
}
// drawing function
void au::LaserTail::specific_draw(){
	vis::Draw::start_line_strip();
		vis::Draw::set_color( rc::laser_c_r(), rc::laser_c_g(), rc::laser_c_b(), 0.0 );
		vis::Draw::add_point( xy );

		parts->specific_draw();
	vis::Draw::end_line_strip();
}
// process function
void au::LaserTail::specific_process( int milliseconds ){
	xy += v*milliseconds;

	parts->specific_process( milliseconds, xy, v );

	if ( parts->empty() )
		add_zombie();
}
