// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef PLAYER_HPP
#define PLAYER_HPP

#include "Automaton.hpp"
#include "../rc/Constants.hpp"
#include "../Input.hpp"

namespace au {

// the interactive class controled by the user
class Player : public Automaton{
	public:
		// ctor
		Player( int id, const util::Vector& xy = util::Vector(rc::ring_middlex(), rc::ring_middley()), int t = 0,
		        int ltime = rc::player_litter_int(), int ftime = Automaton::shot_rate(), real angle = 0.0,
		        bool imm = false );
		// dtor
		virtual ~Player();

		// unserializes the automaton
		static void unserialize( std::istream& in );

	private:
		// collision checking
		int generic_collision();

		// player id
		int id;

		// user input
		const Input *input;

		// time since last littering
		int ltime;
		// time since last fire
		int ftime;

		// what direction is the ship looking at
		real angle;

		// immortality
		bool imm;

	// redefined automaton functions

	public:
		// each collision type for active colliders
		virtual void recieve_collision( int& res, ExaminisApicula *p );
		virtual void recieve_collision( int& res, Lagvard *p );
		virtual void recieve_collision( int& res, Proton *p );
		virtual void recieve_collision( int& res, RedHole *p );
		virtual void recieve_collision( int& res, Scared *p );
		virtual void recieve_collision( int& res, Shielded *p );
		virtual void recieve_collision( int& res, Spawned *p );
		virtual void recieve_collision( int& res, Spawnship *p );
		// serializes the automaton
		virtual void serialize( std::ostream& out );

	protected:
		// drawing function
		virtual void specific_draw();
		// process function
		virtual void specific_process( int milliseconds );
};

}

#endif // Player.hpp
