// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include "../rc/Constants.hpp"
#include "Spawned.hpp"

// ctor
au::Spawned::Spawned( Spawn *swarmer, const util::Vector& xy, real phase, bool play ) :
 Automaton( xy, util::Vector(0.0,0.0), rc::spawned_r() ),
 phase( phase ),
 swarmer( swarmer )
{
	m = rc::spawned_mass();

	start_passivecollider( PLAYER_COL | SHOT_COL | REDHOLE_COL );
	start_enemy( play );
}
// dtor
au::Spawned::~Spawned(){
	if ( dolitter )
		Automaton::create_debris( xy, rc::spawned_litter_r(), rc::spawned_litter_c_r(), rc::spawned_litter_c_g(), rc::spawned_litter_c_b() );
	swarmer->takeout( this );
}
// gets current phase
real au::Spawned::get_phase() const{
	return phase;
}
// sets the phase
void au::Spawned::set_phase( real newphase ){
	phase = newphase;
}
// collision functions
void au::Spawned::send_collision( int& res, Automaton *a ){
	a->recieve_collision( res, this );
}
// scoring points
int au::Spawned::get_killscore() const{
	return rc::spawned_kill_score();
}
