// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include "../rc/Constants.hpp"
#include "../vis/Draw.hpp"
#include "../Main.hpp"
#include "Spawnship.hpp"

// ctor
au::Spawnship::Spawnship( const util::Vector& xy, const util::Vector& v, int t, bool play ) :
 Automaton( xy, v, rc::spawnship_r(), t ),
 dospawn( false )
{
	m = rc::spawnship_mass();

	start_passivecollider( PLAYER_COL | SHOT_COL | REDHOLE_COL );
	start_enemy( play );
	start_gravitylistener();
	start_processable();
	start_pursuiter();
	start_visible( SPAWNSHIP_PRIO );
}
// dtor
au::Spawnship::~Spawnship(){
	if ( dolitter ){
		Automaton::create_debris( xy, rc::spawnship_litter_r(),
		                          rc::spawnship_litter_c_r(), rc::spawnship_litter_c_g(), rc::spawnship_litter_c_b() );
	}

	if ( dospawn )
		Automaton::create_spawned( xy );
}
// the spawnship needs to do the spawn
void au::Spawnship::do_spawn(){
	dospawn = true;
}
// serializes the automaton
void au::Spawnship::serialize( std::ostream& out ){
	out << Automaton::SPAWNSHIP_SERIAL
	    << " " << xy << " " << v << " " << t << std::endl;
}
// unserializes the automaton
void au::Spawnship::unserialize( std::istream& in ){
	util::Vector xy;
	util::Vector v;
	int t;

	in >> xy >> v >> t;
	if ( t < 0 )
		throw Main::UNSERIAL_SPAWNSHIP;

	new Spawnship( xy, v, t, false );
}
// drawing function
void au::Spawnship::specific_draw(){
	vis::Draw::translate( xy );
	if ( t < rc::spawnship_t() )
		vis::Draw::scale( 1.0+(rc::spawnship_t()-t)*rc::spawnship_ds() );
	else
		vis::Draw::rotate( v.angle() ); // look to where it is heading
	vis::Draw::play_record( vis::SPAWNSHIP_ID );
}
// where is the nearest player. itis is an unit vector or zero.
// distance is the distance or REAL_MAX
void au::Spawnship::specific_there( const util::Vector& itis, real distance ){
	v = itis * rc::spawnship_s();
}
// process function
void au::Spawnship::specific_process( int milliseconds ){
	if ( t > rc::spawnship_t() )
		xy += v*milliseconds + g*(milliseconds*rc::non_inertial_time_multiplier());

	checkRing_stop();
}
// collision functions
void au::Spawnship::send_collision( int& res, Automaton *a ){
	a->recieve_collision( res, this );
}
// scoring points
int au::Spawnship::get_killscore() const{
	return rc::spawnship_kill_score();
}
