// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include "../rc/Constants.hpp"
#include "Spring.hpp"

// ctor
au::Spring::Spring( const util::Vector& xy, const util::Vector& v, real alpha ) :
 Automaton( xy, v, 0.0 ),
 o( xy ),
 alpha( alpha )
{
	start_gravitylistener();
	start_processable();
}
// dtor
au::Spring::~Spring(){
}
// gets alpha color component
real au::Spring::get_color_alpha() const{
	return alpha;
}
// postoperations when grid serialization has finished
void au::Spring::end_serialize(){
	start_processable();
}
// serializes the automaton
void au::Spring::serialize( std::ostream& out ){
	end_processable(); // avoids later interference

	out << xy << " " << v
	    << " " << o << " " << alpha << std::endl;
}
// unserializes the automaton
au::Spring *au::Spring::unserialize( std::istream& in ){
	util::Vector xy;
	util::Vector v;
	util::Vector o;
	real alpha;

	in >> xy >> v >> o >> alpha;

	Spring *res = new Spring( xy, v, alpha );
	res->o = o;
	return res;
}
// process function
void au::Spring::specific_process( int milliseconds ){
	util::Vector a = g + (o-xy)*(rc::constant_k()/rc::spring_mass()); // acceleration

	xy += v*milliseconds + a*(milliseconds*milliseconds/2.0);

	v += a * milliseconds;
	v *= 1.0 - rc::spring_s_lose()*milliseconds; // lineally decreasing speed

	// computing alpha
	alpha = v.magnitude() + (xy-o).magnitude()/1000.0;
	if ( alpha > rc::spring_color_magnitude() )
		alpha = 1.0;
	else
		alpha /= rc::spring_color_magnitude();
}
