// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include <sstream>
#include <iomanip>
#include "../rc/Constants.hpp"
#include "../vis/Draw.hpp"
#include "../Locale.hpp"
#include "../Main.hpp"
#include "GUI.hpp"

#define SCORES_FIELD 7

// statetext parts
enum stateparts {
	GUI_PART_SCORE = 0,
	GUI_PART_SCORE2,
	GUI_PART_SPACE1,
	GUI_PART_LIFE1,
	GUI_PART_LIFE2,
	GUI_PART_LIFE3,
	GUI_PART_SPACE2,
	GUI_PART_BOMBS,
	GUI_PART_BOMBS2,
	GUI_PART_SPACE3,
	GUI_PART_HISCORE,
	GUI_PART_HISCORE2,

	GUI_PART_AMOUNT
};

// gui text
std::vector<std::string> gui::GUI::statetext(GUI_PART_AMOUNT);
std::vector<std::string> gui::GUI::gameovertext(1);

// information has been updated
bool gui::GUI::updated;

// what will be printed to screen
vis::Font *gui::GUI::state = NULL;
vis::Font *gui::GUI::gameover = NULL;

// initializes the gui [depends on: Locale]
void gui::GUI::init(){
	statetext[GUI_PART_SCORE]   = Locale::get( Locale::SCORE_STR );
	statetext[GUI_PART_SPACE1]  = "   ";
	statetext[GUI_PART_LIFE1]   = "#";
	statetext[GUI_PART_LIFE2]   = "#";
	statetext[GUI_PART_LIFE3]   = "#";
	statetext[GUI_PART_SPACE2]  = "   ";
	statetext[GUI_PART_BOMBS]   = Locale::get( Locale::BOMBS_STR );
	statetext[GUI_PART_SPACE3]  = "   ";
	statetext[GUI_PART_HISCORE] = Locale::get( Locale::HISCORE_STR );

	gameovertext[0] = Locale::get( Locale::GAMEOVER_STR );

	updated = true;
}
// ends the gui
void gui::GUI::finalize(){
	if ( state != NULL )
		delete state;
	if ( gameover != NULL )
		delete gameover;

	statetext.clear();
	gameovertext.clear();
}
// global update
void gui::GUI::update(){
	updateScore();
	updateGameover();
	updateLifes();
	updateBombs();
	updateHiscore();
}
// updating GUI's score
void gui::GUI::updateScore(){
	std::ostringstream oss;

	oss << std::setfill(' ') << std::setw(SCORES_FIELD) << Main::get_score();

	statetext[GUI_PART_SCORE2] = oss.str();

	updated = true;
}
// updating GUI's game over
void gui::GUI::updateGameover(){
	updated = true;
}
// updating GUI's lifes
void gui::GUI::updateLifes(){
	updated = true;
}
// updating GUI's bombs
void gui::GUI::updateBombs(){
	std::ostringstream oss;

	oss << std::setfill(' ') << std::setw(1) << Main::get_bombs();

	statetext[GUI_PART_BOMBS2] = oss.str();

	updated = true;
}
// updating GUI's hiscore
void gui::GUI::updateHiscore(){
	std::ostringstream oss;

	oss << std::setfill(' ') << std::setw(SCORES_FIELD) << Main::get_hiscore();

	statetext[GUI_PART_HISCORE2] = oss.str();

	updated = true;
}
// initialize drawing
void gui::GUI::initdrawing(){
	if ( state != NULL ){
		delete state;
		state = NULL;
	}
	if ( gameover != NULL ){
		delete gameover;
		gameover = NULL;
	}

	init();

	updated = true;
}
// drawing function
void gui::GUI::draw(){
	vis::Draw::reset();

	if ( updated ){
		updated = false;

		// updating the stats
		if ( state != NULL )
			delete state;
		state = new vis::Font( vis::Font::GUI_TEXT, statetext, &statecolor );

		// updating the game over
		if ( Main::get_gameover() ){
			if ( gameover != NULL )
				delete gameover;
			gameover = new vis::Font( vis::Font::MENU_TEXT, gameovertext, &gameovercolor );
		}else{
			if ( gameover != NULL ){
				delete gameover;
				gameover = NULL;
			}
		}
	}

	if ( state != NULL )
		state->draw();
	if ( gameover != NULL )
		gameover->draw();
}
// stats color callback
void gui::GUI::statecolor( int part ){
	switch(part){
		case GUI_PART_SCORE:
		case GUI_PART_SCORE2:
			vis::Draw::set_color( rc::gui_score_c_r(), rc::gui_score_c_g(), rc::gui_score_c_b() );
			break;
		case GUI_PART_SPACE1:
			vis::Draw::set_color( 0.0, 0.0, 0.0 );
			break;
		case GUI_PART_LIFE1:
		case GUI_PART_LIFE2:
		case GUI_PART_LIFE3:
			if ( (part - GUI_PART_LIFE1) >= Main::get_lifes() )
				vis::Draw::set_color( rc::gui_nolife_c_r(), rc::gui_nolife_c_g(), rc::gui_nolife_c_b(), Main::get_merciless()? 0.0 : 1.0 );
			else
				vis::Draw::set_color( rc::gui_life_c_r(), rc::gui_life_c_g(), rc::gui_life_c_b(), Main::get_merciless()? 0.0 : 1.0 );
			break;
		case GUI_PART_SPACE2:
			vis::Draw::set_color( 0.0, 0.0, 0.0 );
			break;
		case GUI_PART_BOMBS:
		case GUI_PART_BOMBS2:
			vis::Draw::set_color( rc::gui_bombs_c_r(), rc::gui_bombs_c_g(), rc::gui_bombs_c_b() );
			break;
		case GUI_PART_SPACE3:
			vis::Draw::set_color( 0.0, 0.0, 0.0 );
			break;
		case GUI_PART_HISCORE:
		case GUI_PART_HISCORE2:
			vis::Draw::set_color( rc::gui_hiscore_c_r(), rc::gui_hiscore_c_g(), rc::gui_hiscore_c_b() );
			break;
	}
}
// game over color callback
void gui::GUI::gameovercolor( int part ){
	vis::Draw::set_color( rc::gui_gameover_c_r(), rc::gui_gameover_c_g(), rc::gui_gameover_c_b() );
}
