// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef MENU_CONTROLS_HPP
#define MENU_CONTROLS_HPP

#include <map>
#include <vector>
#include "../Config.hpp"
#include "Menu_Component.hpp"

namespace gui {

class Menu_Controls : public Menu_Component{
	public:
		// ctor
		Menu_Controls();
		// dtor
		virtual ~Menu_Controls();

		// controls pane start
		virtual void start( int from );
		// functions to process user events
		virtual void exit();
		virtual void move( int updown );
		virtual void change( int moreless );
		virtual void accept();

		// capture ended
		virtual void notify( bool success, SDLKey key, int joy, int axis_but, bool invert );

	private:
		// controls pane ended
		void end();
		// updates the text
		void update();
		// save controls
		void save();
		// tests if present config is acceptable, that is: has the needed to let the player exit the game
		bool acceptable();

		// sets plcntrl with pertinent info
		void resetplcntrl();

		// gets a string representation of a key
		std::string getname( SDLKey key );
		// gets a string representation of a joystic-axis or a joystick-button
		std::string getname( bool axis, const std::pair<int,int>& joy );

		// id of the player that is being configured
		int playerid;
		// controls
		std::vector<Config::playercontrols> playercntrls;
		// present controls being configured
		Config::playercontrols plcntrl;

		// maps a key to the <playerid,controlid> that uses it
		std::map< SDLKey, std::pair<int,int> > keymappings; // unordered
		// maps a joystick to the <playerid,controlid> that uses it
		std::map< std::pair<int,int>, std::pair<int,int> > buttonmappings; // unordered
		std::map< std::pair<int,int>, std::pair<int,int> > axismappings; // unordered

		// sets up a key and fixes configuration to avoid key remappings
		void addkeymapping( SDLKey key, const std::pair<int,int>& id );
		// sets up a button and fixes configuration to avoid button remappings
		void addbuttonmapping( const std::pair<int,int>& joybut, const std::pair<int,int>& id );
		// sets up an axis and fixes configuration to avoid axis remappings
		void addaxismapping( const std::pair<int,int>& joyaxis, const std::pair<int,int>& id );
};

}

#endif // Menu_Controls.hpp
