// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include "../Main.hpp"
#include "../Locale.hpp"
#include "Menu.hpp"
#include "Menu_Root.hpp"

// line codes
enum root_lines {
	ROOT_LINE_RESTART = 0,
	ROOT_LINE_SKIP0,
	ROOT_LINE_SAVE,
	ROOT_LINE_LOAD,
	ROOT_LINE_CONFIG,
	ROOT_LINE_CONTROLS,
	ROOT_LINE_SKIP1,
	ROOT_LINE_EXIT,

	ROOT_LINE_DEFAULT = ROOT_LINE_CONFIG,

	ROOT_LINE_MIN = ROOT_LINE_RESTART,
	ROOT_LINE_MAX = ROOT_LINE_EXIT
};

// ctor
gui::Menu_Root::Menu_Root(){
	text = std::vector<std::string>(ROOT_LINE_MAX+1);
	text[ROOT_LINE_SKIP0] = " ";
	text[ROOT_LINE_SKIP1] = " ";
}
// dtor
gui::Menu_Root::~Menu_Root(){
}
// root pane start
void gui::Menu_Root::start( int from ){
	// setting present line depending on who started the pane
	switch(from){
		case Menu::CONFIG_PANE:
			line = ROOT_LINE_CONFIG;
			break;
		case Menu::CONTROLS_PANE:
			line = ROOT_LINE_CONTROLS;
			break;
		default:
			line = ROOT_LINE_DEFAULT;
			break;
	}
	update();
}
// root pane ended, no other pane to start
void gui::Menu_Root::end(){
	Menu::start( Menu::NO_PANE );
}
// updates the text
void gui::Menu_Root::update(){
	text[ROOT_LINE_RESTART]  = Locale::get( Locale::ROOT_STR_RESTART );
	text[ROOT_LINE_SAVE]     = Locale::get( Locale::ROOT_STR_SAVE );
	text[ROOT_LINE_LOAD]     = Locale::get( Locale::ROOT_STR_LOAD );
	text[ROOT_LINE_CONFIG]   = Locale::get( Locale::ROOT_STR_CONFIG );
	text[ROOT_LINE_CONTROLS] = Locale::get( Locale::ROOT_STR_CONTROLS );
	text[ROOT_LINE_EXIT]     = Locale::get( Locale::ROOT_STR_EXIT );

	Menu::update();
}
// user event: exit
void gui::Menu_Root::exit(){
	end();
}
// user event: movement
void gui::Menu_Root::move( int updown ){
	// changing line
	line += updown;
	switch(line){
		case ROOT_LINE_SKIP0:
		case ROOT_LINE_SKIP1:
			line += updown;
			break;
	}
	// wrapping
	if ( line < ROOT_LINE_MIN )
		line = ROOT_LINE_MAX;
	else if ( line > ROOT_LINE_MAX )
		line = ROOT_LINE_MIN;
}
// user event: change
void gui::Menu_Root::change( int moreless ){
}
// user event: accept
void gui::Menu_Root::accept(){
	switch(line){
		// restarting game
		case ROOT_LINE_RESTART:
			Main::start();
			end();
			break;
		// saves the present game
		case ROOT_LINE_SAVE:
			Main::serialize();
			break;
		// loads a previously saved game
		case ROOT_LINE_LOAD:
			Main::unserialize();
			break;
		// starting config pane
		case ROOT_LINE_CONFIG:
			Menu::start( Menu::CONFIG_PANE );
			break;
		// starting controls pane
		case ROOT_LINE_CONTROLS:
			Menu::start( Menu::CONTROLS_PANE );
			break;
		// quitting game
		case ROOT_LINE_EXIT:
			Main::quit( Main::NORMAL_EXIT );
			break;
	}
}
// capture ended: ignore
void gui::Menu_Root::notify( bool success, SDLKey key, int joy, int axis_but, bool invert ){
}
