// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

// ring size (integer values)
REAL_VAR( width )
REAL_VAR( height )
// ring coordinates
REAL_VAR( ring_x0 )
REAL_VAR( ring_x1 )
REAL_VAR( ring_y0 )
REAL_VAR( ring_y1 )
// useful positions in the ring
REAL_VAR( ring_middlex )
REAL_VAR( ring_middley )
REAL_VAR( ring_top )
REAL_VAR( ring_bottom )
REAL_VAR( ring_left )
REAL_VAR( ring_right )

// acceleration to velocity transformation
REAL_VAR( non_inertial_time_multiplier )

// the resolution used to group the passive colliders depending on
// their position and radius (i.e. the amount of ring's partitions)
INT_VAR( colliderpassive_resolution )

// automatons' radius (unit: widths)
REAL_VAR( bullet_r )
REAL_VAR( debris_r )
REAL_VAR( examinis_apicula_r )
REAL_VAR( examinis_orbitmin_r )
REAL_VAR( examinis_orbitmax_r )
REAL_VAR( examinis_queen_r )
REAL_VAR( lagvard_r )
REAL_VAR( laser_r )
REAL_VAR( lightningspark_r )
REAL_VAR( player_collision_r ) // radius to check collisions
REAL_VAR( player_ring_r ) // radius to check ring's limits
REAL_VAR( player_shoot_r ) // radius where shots are created
REAL_VAR( proton_r )
REAL_VAR( redhole_r )
REAL_VAR( rubble_r )
REAL_VAR( scared_r )
REAL_VAR( shielded_r )
REAL_VAR( shielded_inner_r )
REAL_VAR( spawn_r ) // radius of the spawned rotating circle
REAL_VAR( spawned_r )
REAL_VAR( spawnship_r )
// automatons' width (unit: widths)
REAL_VAR( blast_w )
REAL_VAR( shielded_w )
// starting scale factor for just born redholes
REAL_VAR( redhole_birth_scale )
REAL_VAR( redhole_inner_scale ) // the small inner redhole when hit

// automatons' speed (unit: widths per millisecond)
REAL_VAR( blast_s )
REAL_VAR( bullet_s )
REAL_VAR( debris_s )
REAL_VAR( examinis_s )
REAL_VAR( fast_bullet_s )
REAL_VAR( lagvard_s )
REAL_VAR( laser_s )
REAL_VAR( laser_tail_s )
REAL_VAR( lightning_s )
REAL_VAR( multi_bullet_s )
REAL_VAR( player_s )
REAL_VAR( proton_s )
REAL_VAR( redhole_s )
REAL_VAR( scared_s )
REAL_VAR( shielded_s )
REAL_VAR( spawn_s )
REAL_VAR( spawnship_s )
// speed losing ratio when collision
REAL_VAR( redhole_s_lose_c )
REAL_VAR( shielded_s_lose_c )
// speed losing ratio (unit: tant-per-one per millisecond)
REAL_VAR( debris_s_lose )
REAL_VAR( rubble_s_lose )
REAL_VAR( spring_s_lose )

// automatons' angular speed (unit: radians per millisecond)
REAL_VAR( examinis_apicula_as )
REAL_VAR( examinis_orbit_as )
REAL_VAR( shielded_as )
REAL_VAR( spawn_as )

// automatons' times (unit: milliseconds)
INT_VAR( blast_t ) // time blast is alive
INT_VAR( examinis_t ) // scale factor of the geometric progression used for apiculas spawn time
INT_VAR( lagvard_t ) // time till lagvard moves
INT_VAR( lightning_t ) // time lightning is alive
INT_VAR( lightning_dead_t ) // time lightning is shown once dead
INT_VAR( player_imm_t ) // player immunity time
INT_VAR( player_reb_t ) // time waiting to rebirth
INT_VAR( redhole_t ) // time the inner redhole is shown
INT_VAR( scared_t ) // time till scared moves
INT_VAR( spawn_t ) // time till spawned's ciruclar formation is fully open
INT_VAR( spawnship_t ) // time till spawnship moves
// size-decreasing speed when spawning (unit: tant-per-one per millisecond)
REAL_VAR( lagvard_ds )
REAL_VAR( scared_ds )
REAL_VAR( spawnship_ds )

// automatons' lifes
INT_VAR( bullet_lifes )
INT_VAR( examinis_apicula_lifes )
INT_VAR( examinis_queen_lifes )
INT_VAR( laser_lifes )
INT_VAR( lightning_lifes )
INT_VAR( lightning_main_lifes )
INT_VAR( redhole_lifes )
INT_VAR( shielded_lifes )

// enemies' score points
INT_VAR( examinis_hit_score )
INT_VAR( examinis_kill_score )
INT_VAR( lagvard_kill_score )
INT_VAR( proton_kill_score )
INT_VAR( redhole_hit_score )
INT_VAR( redhole_kill_score )
INT_VAR( scared_kill_score )
INT_VAR( shielded_hit_score )
INT_VAR( shielded_kill_score )
INT_VAR( spawned_kill_score )
INT_VAR( spawnship_kill_score )

// automatons' masses
REAL_VAR( lagvard_mass )
REAL_VAR( redhole_mass )
REAL_VAR( redhole_mass_i ) // blast interval
REAL_VAR( redhole_mass_m_min ) // min maximum allowed redhole mass
REAL_VAR( redhole_mass_m_max ) // max maximum allowed redhole mass
REAL_VAR( redhole_mass_t ) // no proton spawning threshold
REAL_VAR( scared_mass )
REAL_VAR( spawned_mass )
REAL_VAR( spawnship_mass )
REAL_VAR( spring_mass )
// proton proportion (unit: protons per redhole_mass)
REAL_VAR( proton_proportion )

// shielded penetration angle (unit: radians)
REAL_VAR( shielded_pass_range )
// debris and rubble fading out and vanishing (unit: tant-per-one per millisecond)
REAL_VAR( debris_fade_out )
REAL_VAR( rubble_fade_out )
REAL_VAR( debris_min_visiblity )
REAL_VAR( rubble_min_visiblity )

// bullet drawing magic numbers
REAL_VAR( bullet_litter_c_r )
REAL_VAR( bullet_litter_c_g )
REAL_VAR( bullet_litter_c_b )
REAL_VAR( bullet_litter_r ) // unit: widths
// background color
REAL_VAR( clear_c_r )
REAL_VAR( clear_c_g )
REAL_VAR( clear_c_b )
REAL_VAR( clear_c_a )
// debris line length, computed as where it was some time ago (unit: milliseconds)
INT_VAR( debris_milliseconds_ago )
REAL_VAR( debris_per_radiant ) // debris density
// examinis drawing magic numbers
REAL_VAR( examinis_apicula_c_r )
REAL_VAR( examinis_apicula_c_g )
REAL_VAR( examinis_apicula_c_b )
REAL_VAR( examinis_apicula_litter_c_r )
REAL_VAR( examinis_apicula_litter_c_g )
REAL_VAR( examinis_apicula_litter_c_b )
REAL_VAR( examinis_apicula_litter_r ) // unit: widths
REAL_VAR( examinis_queen_c_r )
REAL_VAR( examinis_queen_c_g )
REAL_VAR( examinis_queen_c_b )
REAL_VAR( examinis_queen_litter_c_r )
REAL_VAR( examinis_queen_litter_c_g )
REAL_VAR( examinis_queen_litter_c_b )
REAL_VAR( examinis_queen_litter_r ) // unit: widths
// gui colors
REAL_VAR( gui_score_c_r )
REAL_VAR( gui_score_c_g )
REAL_VAR( gui_score_c_b )
REAL_VAR( gui_life_c_r )
REAL_VAR( gui_life_c_g )
REAL_VAR( gui_life_c_b )
REAL_VAR( gui_nolife_c_r )
REAL_VAR( gui_nolife_c_g )
REAL_VAR( gui_nolife_c_b )
REAL_VAR( gui_bombs_c_r )
REAL_VAR( gui_bombs_c_g )
REAL_VAR( gui_bombs_c_b )
REAL_VAR( gui_hiscore_c_r )
REAL_VAR( gui_hiscore_c_g )
REAL_VAR( gui_hiscore_c_b )
REAL_VAR( gui_gameover_c_r )
REAL_VAR( gui_gameover_c_g )
REAL_VAR( gui_gameover_c_b )
// lagvard drawing magic numbers
REAL_VAR( lagvard_litter_c_r )
REAL_VAR( lagvard_litter_c_g )
REAL_VAR( lagvard_litter_c_b )
REAL_VAR( lagvard_litter_r ) // unit: widths
// laser color
REAL_VAR( laser_c_r )
REAL_VAR( laser_c_g )
REAL_VAR( laser_c_b )
REAL_VAR( laser_litter_c_r )
REAL_VAR( laser_litter_c_g )
REAL_VAR( laser_litter_c_b )
REAL_VAR( laser_litter_r ) // unit: widths
// lightning colors
REAL_VAR( lightning_c_r )
REAL_VAR( lightning_c_g )
REAL_VAR( lightning_c_b )
REAL_VAR( lightning_main_c_r )
REAL_VAR( lightning_main_c_g )
REAL_VAR( lightning_main_c_b )
// menu colors
REAL_VAR( menu_active_c_r )
REAL_VAR( menu_active_c_g )
REAL_VAR( menu_active_c_b )
REAL_VAR( menu_background_c_r )
REAL_VAR( menu_background_c_g )
REAL_VAR( menu_background_c_b )
REAL_VAR( menu_background_c_a )
REAL_VAR( menu_margin_c_r )
REAL_VAR( menu_margin_c_g )
REAL_VAR( menu_margin_c_b )
REAL_VAR( menu_margin_c_a )
REAL_VAR( menu_normal_c_r )
REAL_VAR( menu_normal_c_g )
REAL_VAR( menu_normal_c_b )
// player drawing magic numbers
INT_VAR( player_defense_blinking_time ) // defense blinking time (unit: milliseconds)
INT_VAR( player_defense_blink_time ) // defense blink time (unit: milliseconds)
INT_VAR( player_litter_int ) // unit: milliseconds
REAL_VAR( player_litter_phi ) // unit: radians
REAL_VAR( player_litter_r ) // unit: widths
REAL_VAR( debris_min_red ) // ranges for random color debris
REAL_VAR( debris_max_red )
REAL_VAR( debris_min_green )
REAL_VAR( debris_max_green )
REAL_VAR( debris_min_blue )
REAL_VAR( debris_max_blue )
// proton drawing magic numbers
REAL_VAR( proton_litter_c_r )
REAL_VAR( proton_litter_c_g )
REAL_VAR( proton_litter_c_b )
REAL_VAR( proton_litter_r ) // unit: widths
// redhole drawing magic numbers
REAL_VAR( redhole_c_r )
REAL_VAR( redhole_c_g )
REAL_VAR( redhole_c_b )
REAL_VAR( redhole_inner_c_r )
REAL_VAR( redhole_inner_c_g )
REAL_VAR( redhole_inner_c_b )
REAL_VAR( redhole_litter_c_r )
REAL_VAR( redhole_litter_c_g )
REAL_VAR( redhole_litter_c_b )
// rubble line length, computed as where it was some time ago (unit: milliseconds)
INT_VAR( rubble_milliseconds_ago )
// scared drawing magic numbers
REAL_VAR( scared_litter_c_r )
REAL_VAR( scared_litter_c_g )
REAL_VAR( scared_litter_c_b )
REAL_VAR( scared_litter_r ) // unit: widths
// shielded drawing magic numbers
REAL_VAR( shielded_c_r )
REAL_VAR( shielded_c_g )
REAL_VAR( shielded_c_b )
REAL_VAR( shielded_inner_c_r )
REAL_VAR( shielded_inner_c_g )
REAL_VAR( shielded_inner_c_b )
REAL_VAR( shielded_litter_c_r )
REAL_VAR( shielded_litter_c_g )
REAL_VAR( shielded_litter_c_b )
REAL_VAR( shielded_litter_r ) // unit: widths
REAL_VAR( shielded_litter_inner_r ) // unit: widths
// spawned drawing magic numbers
REAL_VAR( spawned_litter_c_r )
REAL_VAR( spawned_litter_c_g )
REAL_VAR( spawned_litter_c_b )
REAL_VAR( spawned_litter_r ) // unit: widths
// spawnship drawing magic numbers
REAL_VAR( spawnship_litter_c_r )
REAL_VAR( spawnship_litter_c_g )
REAL_VAR( spawnship_litter_c_b )
REAL_VAR( spawnship_litter_r ) // unit: widths
// grid dots' control points distance when using bezier curves.
// the distance between consecutive columns is 1.0
REAL_VAR( grid_control_point_r )
// grid color gradient
// the higher it is, the darker is the grid (more gravity to see it)
REAL_VAR( spring_color_magnitude )
REAL_VAR( spring_c_r )
REAL_VAR( spring_c_g )
REAL_VAR( spring_c_b )

// shot score point intervals
INT_VAR( normal_shots_int )
INT_VAR( fast_shots_int )
INT_VAR( multi_shots_int )
INT_VAR( laser_shots_int )
INT_VAR( lightning_shots_int )
// shot rates (unit: milliseconds)
INT_VAR( normal_shots_rate )
INT_VAR( fast_shots_rate )
INT_VAR( multi_shots_rate )
INT_VAR( laser_shots_rate )
INT_VAR( lightning_shots_rate )
// maximum angle deviation on multi shots (unit: radians)
INT_VAR( multi_shots_amount )
REAL_VAR( multi_shots_range )
// lightning shots parameters
REAL_VAR( lightning_split_min ) // time range between splits
REAL_VAR( lightning_split_max )
REAL_VAR( lightning_split_angle ) // angle the child spark can diverge, in radians
REAL_VAR( lightning_turn_min ) // time range between small direction changes
REAL_VAR( lightning_turn_max )
REAL_VAR( lightning_drift ) // sparks' direction change
REAL_VAR( lightning_head_drift ) // main spark's direction change

// power ups defined by score increments
INT_VAR( powerup_lifes )
INT_VAR( powerup_bombs )

// enemy spawning areas' radius (unit: widths)
REAL_VAR( big_spawning_area_r )
REAL_VAR( small_spawning_area_r )
// enemies minimum number
INT_VAR( enemy_threshold )
// enemy spawning interval (unit: milliseconds)
INT_VAR( peace_time )
// game activity heuristic computation: pl*sc*se where
REAL_VAR( player_increment ) // pl = 1.0 + (players-1)*player_increment
REAL_VAR( score_revamping ) // sc = 1.0 + score*score_revamping
REAL_VAR( game_overwhelming ) // se = min( 1.0, game_overwhelming*(shots/enemies) )
REAL_VAR( min_activity )
REAL_VAR( max_activity )
// spawn probabilities (unit: any value, they are normalized later)
REAL_VAR( examinis_prob )
REAL_VAR( lagvard_prob )
REAL_VAR( redhole_prob )
REAL_VAR( scared_prob )
REAL_VAR( shielded_prob )
REAL_VAR( spawnship_prob )
REAL_VAR( no_spawning_prob ) // usually it should be 0
// spawn probabilities update ratio
INT_VAR( prob_variation_int )
REAL_VAR( examinis_prob_variation )
REAL_VAR( lagvard_prob_variation )
REAL_VAR( redhole_prob_variation )
REAL_VAR( scared_prob_variation )
REAL_VAR( shielded_prob_variation )
REAL_VAR( spawnship_prob_variation )
REAL_VAR( no_spawning_prob_variation )
// which enemies are spawned just once at a time
BOOL_VAR( examinis_single )
BOOL_VAR( lagvard_single )
BOOL_VAR( redhole_single )
BOOL_VAR( scared_single )
BOOL_VAR( shielded_single )
BOOL_VAR( spawnship_single )
// time the spawning process lasts (unit: milliseconds)
INT_VAR( examinis_time )
INT_VAR( lagvard_time )
INT_VAR( redhole_time )
INT_VAR( scared_time )
INT_VAR( shielded_time )
INT_VAR( spawnship_time )
// time between consecutive spawns (unit: milliseconds)
INT_VAR( examinis_int )
INT_VAR( lagvard_int )
INT_VAR( redhole_int )
INT_VAR( scared_int )
INT_VAR( shielded_int )
INT_VAR( spawnship_int )
// which enemies spawn from random locations
BOOL_VAR( examinis_rand )
BOOL_VAR( lagvard_rand )
BOOL_VAR( redhole_rand )
BOOL_VAR( scared_rand )
BOOL_VAR( shielded_rand )
BOOL_VAR( spawnship_rand )

// forces
REAL_VAR( constant_k )
REAL_VAR( constant_g )
REAL_VAR( max_g )
// gravity field that keeps debris alive
REAL_VAR( debris_g_threshold )
// scared dodging constants
REAL_VAR( scared_fear_i )
REAL_VAR( scared_fear_r )
REAL_VAR( max_fear )
// movement done in each colliding object to avoid the intersection, 0.5 would be the most realistic
REAL_VAR( collision_guiltyness )

// drawing line widths
REAL_VAR( normal_line_width )
REAL_VAR( thin_line_width )
// heuristic computation of needed points in a bezier curve
REAL_VAR( bezier_heuristic ) // amount of points for every pixel when considering straight lines
REAL_VAR( bezier_max )
REAL_VAR( visible_alpha_threshold )

// menu interaction times (unit: milliseconds)
INT_VAR( menu_response_time )
INT_VAR( joystick_capturing_time )

// values to define the interval for non real-time computations:
//   a negative value is the amount of processing iterations
//   a positive value is the amount of milliseconds
//   0 for real time
INT_VAR( gravity_int )
INT_VAR( pursuit_int )
INT_VAR( dodging_int )
INT_VAR( collide_int )

// the font's rendering point size
INT_VAR( font_point_size )
