// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include <cctype>
#include <boost/foreach.hpp>
#define foreach BOOST_FOREACH
#include "Text.hpp"

#define UTF8_EXTRABYTE      0x80
#define UTF8_EXTRABYTE_MASK 0xC0

#define UTF8_BOM_BYTE0 0xEF
#define UTF8_BOM_BYTE1 0xBB
#define UTF8_BOM_BYTE2 0xBF
#define UTF8_BOM_MASK  0xFF

// returns a copy of str with all leading and trailing white space
// characters removed
std::string util::Text::trim( const std::string& str ){
	std::string res(str);

	// erasing leading white space
	while( !res.empty() && std::isspace(res[0]) )
		res.erase( 0, 1 );
	// erasing trailing white space
	while( !res.empty() && std::isspace(res[res.size()-1]) )
		res.erase( res.size()-1, 1 );

	return res;
}
// returns a copy of str with the first line feed character (or the first
// carriage return character) and on truncated.
std::string util::Text::eraseNewLine( const std::string& str ){
	std::string res(str);
	std::string::size_type i;

	// line feed
	i = res.find( '\012' );
	if ( i != std::string::npos )
		res.erase( i );

	// carriage return
	i = res.find( '\015' );
	if ( i != std::string::npos )
		res.erase( i );

	return res;
}
// returns a copy of str with the first three bytes skipped if they are
// the UTF-8's BOM
std::string util::Text::utf8SkipBOM( const std::string& str ){
	const char *aux = str.c_str();
	return std::string( ( ((aux[0] & UTF8_BOM_MASK) == UTF8_BOM_BYTE0)
	                      && ((aux[1] & UTF8_BOM_MASK) == UTF8_BOM_BYTE1)
	                      && ((aux[2] & UTF8_BOM_MASK) == UTF8_BOM_BYTE2) ) ? &aux[3] : aux );
}
// returns the total number of extra chars needed to form the Unicode
// characteres in str (i.e.: str.length() - extra == num of Unicode characteres)
// note: it does not take into account ligatures and the likes
int util::Text::utf8OverheadBytes( const std::string& str ){
	int count = 0;

	foreach( std::string::const_reference i, str ){
		if ( ( i & UTF8_EXTRABYTE_MASK ) == UTF8_EXTRABYTE )
			++count;
	}

	return count;
}
