// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef FONT_HPP
#define FONT_HPP

#include <list>
#include <vector>
#include <string>
#include <utility>
#include <ftgl.h>
#include "../util/Real.hpp"

namespace vis {

class Font{
	public:
		// initializes the font [depends on: Constants]
		static void init();
		// ends the font
		static void finalize();

		// types of the text to render:
		//   gui text: top, from left most to right most
		//   menu: centered
		//   menu framed: centered and framed
		enum texttypes {
			GUI_TEXT = 0,
			MENU_TEXT,
			MENU_TEXT_FRAMED
		};

	private:
		// font face
		static FTFont *font;

	// non-static part of Font

	public:
		// ctor, specifying the texttype, the text to render and the color callback
		Font( int type, const std::vector<std::string>& text, void (*color) (int) );
		// dtor
		virtual ~Font();

		// drawing function
		void draw();

	private:
		// draw a frame? and its coordinates
		bool frame;
		real bottom;
		real top;
		real left;
		real right;
		// color callback
		void (*color) (int);
		// text parts' pen position
		std::list< std::pair<real,real> > parts;
		// rendering scale
		real scale;
		// the text that has to be rendered
		std::list<std::string> text;
};

}

#endif // Font.hpp
